@@ -10,9 +10,9 @@ use axum::{
     AddExtensionLayer, Router,
 };
 use rustdb::{
-    c_value, check_types, standard_builtins, AccessPagedData, AtomicFile, Block, BuiltinMap, CExp,
-    CExpPtr, CompileFunc, DataKind, Database, EvalEnv, Expr, GenTransaction, Part, SharedPagedData,
-    SimpleFileStorage, Value, INITSQL,
+    c_value, check_types, expr::ObjRef, standard_builtins, AccessPagedData, AtomicFile, Block,
+    BuiltinMap, CExp, CExpPtr, CompileFunc, DataKind, Database, EvalEnv, Expr, GenTransaction,
+    Part, SharedPagedData, SimpleFileStorage, Value, INITSQL, Transaction
 };
 use std::{collections::BTreeMap, rc::Rc, sync::Arc, thread};
 use tokio::sync::{mpsc, oneshot};
@@ -21,12 +21,14 @@ use tower_cookies::{CookieManagerLayer, Cookies};
 
 /// Transaction to be sent to server thread, implements IntoResponse.
 struct ServerTrans {
+    pub sql: String,
     pub x: Box<GenTransaction>,
 }
 
 impl ServerTrans {
     pub fn new() -> Self {
         Self {
+            sql: "EXEC web.Main()".to_string(),
             x: Box::new(GenTransaction::new()),
         }
     }
@@ -38,6 +40,19 @@ struct ServerMessage {
     pub tx: oneshot::Sender<ServerTrans>,
 }
 
+/// Extra transaction data.
+#[derive(Default)]
+struct TransExt {
+    /// Signals there is new email to be sent.
+    pub email_tx: bool,
+}
+
+impl TransExt {
+    pub fn new() -> Box<Self> {
+        Box::new(Self::default())
+    }
+}
+
 /// State shared with handlers.
 struct SharedState {
     /// Sender channel for sending queries to server thread.
@@ -46,6 +61,7 @@ struct SharedState {
     spd: Arc<SharedPagedData>,
     /// Map of builtin SQL functions for Database.
     bmap: Arc<BuiltinMap>,
+    email_tx: mpsc::Sender<()>,
 }
 
 #[tokio::main]
@@ -66,7 +82,10 @@ async fn main() {
     // Include the Argon hash function as well as the standard functions.
     let mut bmap = BuiltinMap::new();
     standard_builtins(&mut bmap);
-    let list = [("ARGON", DataKind::Binary, CompileFunc::Value(c_argon))];
+    let list = [
+        ("ARGON", DataKind::Binary, CompileFunc::Value(c_argon)),
+        ("EMAILTX", DataKind::Int, CompileFunc::Int(c_email_tx)),
+    ];
     for (name, typ, cf) in list {
         bmap.insert(name.to_string(), (typ, cf));
     }
@@ -78,12 +97,14 @@ async fn main() {
     // Construct thread communication channels.
     let (tx, mut rx) = mpsc::channel::<ServerMessage>(1);
     let (log_tx, log_rx) = std::sync::mpsc::channel::<String>();
+    let (email_tx, email_rx) = mpsc::channel::<()>(1);
 
     // Construct shared state.
     let ss = Arc::new(SharedState {
         tx,
         spd,
         bmap: bmap.clone(),
+        email_tx
     });
 
     // Start the logging thread (synchronous)
@@ -91,12 +112,16 @@ async fn main() {
         log_loop(log_rx);
     });
 
+    // Start the email thread (asynchronous)
+    let email_ss = ss.clone();
+    tokio::spawn(async move { email_loop(email_rx, email_ss).await });
+
     // Start the server thread that updates the database (synchronous).
     thread::spawn(move || {
         let db = Database::new(wapd, INITSQL, bmap);
         loop {
             let mut sm = rx.blocking_recv().unwrap();
-            db.run_timed("EXEC web.Main()", &mut *sm.st.x);
+            db.run_timed(&sm.st.sql, &mut *sm.st.x);
             let updates = db.save();
             if updates > 0 {
                 println!("Pages updated={}", updates);
@@ -130,17 +155,17 @@ async fn h_get(
     cookies: Cookies,
 ) -> ServerTrans {
     // Build the ServerTrans.
-    let mut sq = ServerTrans::new();
-    sq.x.qy.path = path.0;
-    sq.x.qy.params = params.0;
-    sq.x.qy.cookies = map_cookies(cookies);
+    let mut st = ServerTrans::new();
+    st.x.qy.path = path.0;
+    st.x.qy.params = params.0;
+    st.x.qy.cookies = map_cookies(cookies);
 
     let blocking_task = tokio::task::spawn_blocking(move || {
         // GET requests should be read-only.
         let apd = AccessPagedData::new_reader(state.spd.clone());
         let db = Database::new(apd, "", state.bmap.clone());
-        db.run_timed("EXEC web.Main()", &mut *sq.x);
-        sq
+        db.run_timed(&st.sql, &mut *st.x);
+        st
     });
     blocking_task.await.unwrap()
 }
@@ -156,6 +181,7 @@ async fn h_post(
 ) -> ServerTrans {
     // Build the Server Transaction.
     let mut st = ServerTrans::new();
+    st.x.ext = TransExt::new();
     st.x.qy.path = path.0;
     st.x.qy.params = params.0;
     st.x.qy.cookies = map_cookies(cookies);
@@ -167,7 +193,17 @@ async fn h_post(
     // Send transaction to database thread ( and get it back ).
     let (tx, rx) = oneshot::channel::<ServerTrans>();
     let _err = state.tx.send(ServerMessage { st, tx }).await;
-    let result = rx.await.unwrap();
+    let mut result = rx.await.unwrap();
+
+    {
+      let mut ext = result.x.get_extension();
+      if let Some(ext) = ext.downcast_mut::<TransExt>() 
+      {
+        if ext.email_tx {
+          let _err = state.email_tx.send(()).await;
+        }
+      }
+    }
     result
 }
 
@@ -194,6 +230,81 @@ impl IntoResponse for ServerTrans {
     }
 }
 
+/// thread that logs write transactions to a file.
+fn log_loop(rx: std::sync::mpsc::Receiver<String>) {
+    use std::{fs::OpenOptions, io::Write};
+    let filename = "../test.logfile";
+    let mut logfile = OpenOptions::new()
+        .append(true)
+        .create(true)
+        .open(filename)
+        .unwrap();
+    loop {
+        let logstr = rx.recv().unwrap();
+        let _err = logfile.write_all(logstr.as_bytes());
+        let _err = logfile.write_all(b"\r\n");
+    }
+}
+
+// thread that sends emails
+async fn email_loop(mut rx: mpsc::Receiver<()>, state: Arc<SharedState>) {
+    loop {
+        let mut sent = Vec::new();
+        {
+            let _ = rx.recv().await;
+            let apd = AccessPagedData::new_reader(state.spd.clone());
+            let db = Database::new(apd, "", state.bmap.clone());
+            let qt = db.get_table(&ObjRef::new("email", "Queue")).unwrap();
+
+            let keys = Vec::new();
+            for (pp, off) in qt.scan_keys(&db, keys, 0) {
+                let p = &pp.borrow();
+                let a = qt.access(p, off);
+                let msg = a.int(0) as u64;
+                let st = a.int(1);
+
+                let now = std::time::SystemTime::now()
+                    .duration_since(std::time::SystemTime::UNIX_EPOCH)
+                    .unwrap();
+                let now = now.as_micros() as i64 + 62135596800000000; // Per date.Ticks
+
+                println!(
+                    "Queued email id ={} seconds till send={}",
+                    msg,
+                    (st - now) / 1000000
+                );
+                let mt = db.get_table(&ObjRef::new("email", "Msg")).unwrap();
+                let (pp, off) = mt.id_get(&db, msg).unwrap();
+                let p = &pp.borrow();
+                let a = mt.access(p, off);
+                let from = a.str(&db, 0);
+                let to = a.str(&db, 1);
+                let title = a.str(&db, 2);
+                let body = a.str(&db, 3);
+                println!(
+                    "Email from={} to={} title={} body={}",
+                    from, to, title, body
+                );
+                // Actual sending of email not yet implemented...
+                sent.push(msg);
+            }
+        }
+        for msg in sent
+        {
+          email_sent(&state,msg).await;
+        }
+    }
+}
+
+async fn email_sent(state: &SharedState, msg: u64) {
+    let mut st = ServerTrans::new();
+    st.sql = "EXEC email.Sent(".to_string() + &msg.to_string() + ")";
+    // Send transaction to database thread ( and get it back ).
+    let (tx, rx) = oneshot::channel::<ServerTrans>();
+    let _err = state.tx.send(ServerMessage { st, tx }).await;
+    let _result = rx.await.unwrap();
+}
+
 /////////////////////////////////////////////
 // Helper functions for building ServerTrans.
 
@@ -269,17 +380,21 @@ impl CExp<Value> for Argon {
     }
 }
 
-use std::{fs::OpenOptions, io::Write};
-fn log_loop(log_rx: std::sync::mpsc::Receiver<String>) {
-    let filename = "../test.logfile";
-    let mut logfile = OpenOptions::new()
-        .append(true)
-        .create(true)
-        .open(filename)
-        .unwrap();
-    loop {
-        let logstr = log_rx.recv().unwrap();
-        let _err = logfile.write_all(logstr.as_bytes());
-        let _err = logfile.write_all(b"\r\n");
+/// Compile call to EMAILTX.
+fn c_email_tx(b: &Block, args: &mut [Expr]) -> CExpPtr<i64> {
+    check_types(b, args, &[]);
+    Box::new(EmailTx {})
+}
+
+/// Compiled call to EMAILTX
+struct EmailTx {}
+impl CExp<i64> for EmailTx {
+    fn eval(&self, ee: &mut EvalEnv, _d: &[u8]) -> i64 {
+        let mut ext = ee.tr.get_extension();
+        if let Some(mut ext) = ext.downcast_mut::<TransExt>() {
+            ext.email_tx = true;
+        }
+        ee.tr.set_extension(ext);
+        0
     }
 }
