@@ -1,4 +1,4 @@
-use crate::{panic, BTreeMap, Data, Rc, Transaction, Value};
+use crate::{panic, Any, BTreeMap, Data, Rc, Transaction, Value};
 
 use serde::{Deserialize, Serialize};
 
@@ -25,6 +25,7 @@ pub struct GenResponse {
 pub struct GenTransaction {
     pub qy: GenQuery,
     pub rp: GenResponse,
+    pub ext: Box<dyn Any + Send + Sync>,
 }
 
 /// Part of multipart data ( uploaded files ).
@@ -61,6 +62,7 @@ impl GenTransaction {
                 status_code,
                 headers,
             },
+            ext: Box::new(()),
         }
     }
     /// Append string to output.
@@ -166,9 +168,18 @@ impl Transaction for GenTransaction {
         };
         Rc::new(result.to_string())
     }
+
     fn file_content(&mut self, k: i64) -> Data {
         self.qy.parts[k as usize].data.clone()
     }
+
+    fn set_extension(&mut self, ext: Box<dyn Any + Send + Sync>) {
+        self.ext = ext;
+    }
+
+    fn get_extension(&mut self) -> Box<dyn Any + Send + Sync> {
+        std::mem::replace(&mut self.ext, Box::new(0))
+    }
 }
 
 impl Default for GenTransaction {
