@@ -94,6 +94,7 @@
 //!
 //! Work on improving/testing SQL code, browse schema, float I/O. Login.
 //!
+//! Error handling for out of disk space, out of memory etc.
 
 pub use crate::{
     atomfile::AtomicFile,
@@ -138,6 +139,7 @@ use crate::{
 };
 
 use std::{
+    any::Any,
     cell::{Cell, RefCell},
     cmp::Ordering,
     collections::{BTreeMap, BTreeSet, HashMap, HashSet},
@@ -567,7 +569,7 @@ impl TableBuilder {
 }
 
 /// Input/Output message. Query and Response.
-pub trait Transaction: std::any::Any {
+pub trait Transaction {
     /// STATUSCODE builtin function. sets the response status code.
     fn status_code(&mut self, _code: i64) {}
 
@@ -604,6 +606,14 @@ pub trait Transaction: std::any::Any {
     fn get_error(&mut self) -> String {
         String::new()
     }
+
+    /// Set the extension.
+    fn set_extension(&mut self, _ext: Box<dyn Any + Send + Sync>) {}
+
+    /// Get the extension. Note: this takes ownership, so extension needs to be set afterwards.
+    fn get_extension(&mut self) -> Box<dyn Any + Send + Sync> {
+        Box::new(())
+    }
 }
 
 /// Query where output is printed to console (used for initialisation ).
