@@ -1,4 +1,4 @@
-use crate::{panic, util, HashMap, Rc, Transaction, Value};
+use crate::{panic, util, Any, HashMap, Rc, Transaction, Value};
 use std::{io::Read, io::Write, net::TcpStream};
 
 /// Response content is accumulated in output.
@@ -14,6 +14,7 @@ pub struct WebTransaction {
     pub status_code: String,
     pub headers: String,
     pub now: i64, // Micro-seconds since January 1, 1970 0:00:00 UTC
+    pub ext: Box<dyn Any + Send + Sync>,
 }
 
 /// Map for query, form, cookies.
@@ -75,6 +76,7 @@ impl WebTransaction {
             parts,
             err: String::new(),
             now,
+            ext: Box::new(()),
         })
     }
     pub fn trace(&self) {
@@ -178,6 +180,14 @@ impl Transaction for WebTransaction {
         let hdr = name.to_string() + ": " + value + "\r\n";
         self.headers.push_str(&hdr);
     }
+
+    fn set_extension(&mut self, ext: Box<dyn Any + Send + Sync>) {
+        self.ext = ext;
+    }
+
+    fn get_extension(&mut self) -> Box<dyn Any + Send + Sync> {
+        std::mem::replace(&mut self.ext, Box::new(0))
+    }
 }
 /// Parser for http request.
 ///
